function [lamopt, x, errest, lam, X] = errestik( A, b, H, lambdarange, ...
						estype, opts, U, s, V, debug)
%ERRESTIK optimal lambda value in Tikhonov regularization via error estimation
%
%   lamopt = errestik(A,b,H,lambdarange,estype,opts,U,s,V) returns the lambda
%   value which minimizes the error estimate for the solution of a linear
%   system Ax=b by Tikhonov regularization, performing the computation by
%   SVD/GSVD. The search for the optimal lambda is performed by a bisection
%   method.
% 
%   The function requires the matrix A, the right hand side vector b and a
%   regularization matrix H. If H is missing/empty, the identity matrix is
%   used. lambdarange is a 2-vector containing the range for the regularization
%   parameter (default [1e-30,1e2]).
%   estype is the index of the error estimate (default 3).
%   opts may contain various options (empty for defaults, see below).
%   (U,s,V) is the compact SVD of A or GSVD of (A,H) (as generated by csvd or
%   cgsvd).
%
%   lamopt = errestik(A,b,H,lambdarange,estype,opts,P) does the same performing
%   the computation via the pcg function of Matlab.
%   The parameter P, if given, is a preconditioner.
%
%   lamopt = errestik(A,b,H,lambdarange,estype,opts,L,U) uses a factorized
%   preconditioner P=LU.
%
%   [lamopt,x,errest,lambda,X] = errestik(...) returns also the solution
%   corresponding to lamopt, two vectors with the error estimates and lambda
%   values scanned by the algorithm and a matrix whose columns are the
%   corresponding Tikhonov solutions.
%
%   By default, errestik uses estimate enutik for a square system and etanutik
%   for a rectangular one.
%
%   Options: the vector opts may contain various options
%   opts(1)=tau, tolerance for minimum search (default 1e-2)
%   opts(2)=kmax, max number of iterations for minimum search (default 64)
%   opts(3)=forceta, if nonzero forces use of etanutik also for square systems
%
%   See also enutik, etanutik.

% The algorithm used in this function is describe in Section 3.2 of the paper
%   C. Brezinski, G. Rodriguez, and S. Seatzu.
%   Error estimates for the regularization of least squares problems.
%   Numer. Algorithms, 2009. In press. 

%	Giuseppe Rodriguez,  10-03-11
%	Email: rodriguez@unica.it

if (nargin < 2)
	error( 'Too few parameters')
end

if nargin < 10				% show intermediate results
	debug = 0;
end

if nargin < 9				% use svd or pcg
	usesvd = 0;
else
	usesvd = 1;
end

if nargin < 8				% no factorized preconditioner
	s = [];
end

if nargin < 7				% no preconditioner
	U = [];
end

if (nargin < 6) | isempty(opts)		% use default options
	opts = [];
end

if (nargin < 5) | isempty(estype)
	estype = 3;			% error estimate type
end

if (nargin < 4) | isempty(lambdarange)
	lambdarange = [1e-30;1e2];	% default lambda range 
end

if (nargin < 3) | isempty(H)
	H = speye(size(A,2));		% default regularization matrix
end

% set default options
if length(opts) < 3
	forceta = 0;			% do not force use of etanutik
else
	forceta = opts(3);
end
if length(opts) < 2
	kmax = 64;			% max number of iterations
else
	kmax = opts(2);
end
if length(opts) < 1
	tau = 1e-2;			% default accuracy
else
	tau = opts(1);
end

if ~exist('tikhonov','file')
	error('The "Regularization Tools" toolbox is not on the path.')
end

[m n] = size(A);
[p n1] = size(H);
if (n ~= n1) || (m ~= size(b,1))
	error('Size of arrays do not match.')
end
if m == n
	square = 1;
else
	square = 0;
end

if square && forceta
	warning('errestools:errestik:ForceUseOfEtanutik', ...
			'Forcing use of etanutik for a square matrix')
	square = 0;
end

lambdarange = sort(lambdarange(:));
llamfirst = log10(lambdarange(2));
llamlast = log10(lambdarange(1));
nlambda = 5;
lstep = (llamlast-llamfirst)/(nlambda-1);
llam = llamfirst + lstep*[0:nlambda-1]';	% log-equispaced sampling

if usesvd
	[X rho1 eta1] = tikhonov(U,s,V,b,10.^llam);
else
	X = zeros(size(A,2),nlambda);
	tikmatvet = @(x,A,H,lam)A'*(A*x)+(lam*lam)*(H'*(H*x));
	tol = 1e-14;		% stop tolerance for pcg
	maxit = 100;		% max n. of iterations for pcg
	for i = 1:nlambda
		X(:,i) = pcg( @(x)tikmatvet(x,A,H,10^llam(i)),A'*b, ...
							tol,maxit,U,s);
	end
end
R = b*ones(1,nlambda) - A*X;
EX = H'*(H*X);
res = zeros(nlambda,1);
phi = zeros(nlambda,1);
psi = zeros(nlambda,1);
for i = 1:nlambda
	res(i) = norm(R(:,i));
	if square
		phi(i) = R(:,i)'*EX(:,i);
		psi(i) = norm(EX(:,i));
	else
		phi(i) = norm(EX(:,i));
		psi(i) = norm(A*EX(:,i));
	end
end
if square
	errest = enutik( res, phi, psi, 10.^llam, estype);
else
	errest = etanutik( res, phi, psi, 10.^llam, estype);
end
[mm index] = min(errest);

width = abs(llam(1)-llam(2));
k = 0;
while (width > tau) & (nlambda < kmax)
	k = k+1;
	if index > 1
		ll1 = (llam(index)+llam(index-1))/2;
		if usesvd
			[x1 res1 nhx1] = tikhonov(U,s,V,b,10.^ll1);
		else
			x1 = pcg( @(x)tikmatvet(x,A,H,10^ll1),A'*b, ...
							tol,maxit,U,s);
		end
		resv = b-A*x1;
		exv = H'*(H*x1);
		res1 = norm(resv);
		if square
			phi1 = resv'*exv;
			psi1 = norm(exv);
			err1 = enutik( res1, phi1, psi1, 10.^ll1, estype);
		else
			phi1 = norm(exv);
			psi1 = norm(A*exv);
			err1 = etanutik( res1, phi1, psi1, 10.^ll1, estype);
		end
		llam = [llam(1:index-1);ll1;llam(index:end)];
		res = [res(1:index-1);res1;res(index:end)];
		phi = [phi(1:index-1);phi1;phi(index:end)];
		psi = [psi(1:index-1);psi1;psi(index:end)];
		errest = [errest(1:index-1);err1;errest(index:end)];
		X = [X(:,1:index-1) x1 X(:,index:end)];
		nlambda = nlambda + 1;
		index = index+1;
		width = min([width abs(llam(index)-llam(index-1))]);
	end
	if index < nlambda
		ll2 = (llam(index)+llam(index+1))/2;
		if usesvd
			[x2 res2 nhx2] = tikhonov(U,s,V,b,10.^ll2);
		else
			x2 = pcg( @(x)tikmatvet(x,A,H,10^ll2),A'*b, ...
							tol,maxit,U,s);
		end
		resv = b-A*x2;
		exv = H'*(H*x2);
		res2 = norm(resv);
		if square
			phi2 = resv'*exv;
			psi2 = norm(exv);
			err2 = enutik( res2, phi2, psi2, 10.^ll2, estype);
		else
			phi2 = norm(exv);
			psi2 = norm(A*exv);
			err2 = etanutik( res2, phi2, psi2, 10.^ll2, estype);
		end
		llam = [llam(1:index);ll2;llam(index+1:end)];
		res = [res(1:index);res2;res(index+1:end)];
		phi = [phi(1:index-1);phi2;phi(index:end)];
		psi = [psi(1:index);psi2;psi(index+1:end)];
		errest = [errest(1:index);err2;errest(index+1:end)];
		X = [X(:,1:index) x2 X(:,index+1:end)];
		nlambda = nlambda + 1;
		width = min([width abs(llam(index)-llam(index+1))]);
	end
	[mm index] = min(errest);
	if debug
		fprintf( 'k=%g, index=%g, nlambda=%g, width=%g\n', ...
			k, index, nlambda, width);
		[10.^llam errest]
		%pause
	end
end
if nlambda >= kmax
	warning('errestools:errestik:MaxEvaluations', ...
		'Max number of solution evaluations reached')
end

lamopt = 10.^llam(index);
if nargout > 2
	lam = 10.^llam;
end
x = X(:,index);

