function [kopt, x, errest, X] = errestsv( A, b, H, estype, opts, U, s, V, Z, ZI)
%ERRESTSV optimal k value in TSVD via error estimation
%
%   kopt = errestsv(A,b,H,estype,opts) returns the k value which minimizes the
%   error estimate for the solution of a linear system Ax=b by truncated
%   (generalized) singular value decomposition (TSVD/TGSVD).
% 
%   The function requires the matrix A, the right hand side vector b and a
%   regularization matrix H. If H is missing, the identity matrix is used.
%   estype contains the index of the error estimate (default 3).
%   opts may contain various options (empty for defaults, see below).
%
%   If H is the identity matrix, it is possible to give in input the compact
%   SVD of A to save time, if it is available, with the following call
%      [U,s,V] = csvd(A);
%      kopt = errestsv(A,b,H,estype,opts,U,s,V) 
%   The same is true for the compact generalized SVD (GSVD) of (A,H)
%      [U,s,Z,V,ZI] = cgsvd(A,H);
%      kopt = errestsv(A,b,H,estype,opts,U,s,V,Z,ZI) 
%
%   [kopt,x,errest,X] = errestsv(...) returns also the solution corresponding
%   to kopt, a vector with the error estimates and a matrix whose columns
%   contain the TSVD/TGSVD solutions.
%
%   By default, errestsv uses estimate enu for a square system and etanu
%   for a rectangular one.
%
%   Options: opts contains the following optional parameter
%   opts(1): if 1 forces use of etanu estimate also for square systems, if 2
%      applies enu estimate to normal equations
%
%   See also enu, etanu.

% The algorithm used in this function is describe in Section 3.1 of the paper
%   C. Brezinski, G. Rodriguez, and S. Seatzu.
%   Error estimates for the regularization of least squares problems.
%   Numer. Algorithms, 2009. In press. 

%	Giuseppe Rodriguez,  10-03-11
%	Email: rodriguez@unica.it

if (nargin < 2)
	error( 'Too few parameters')
end

if (nargin < 5) | isempty(opts)		% use default options
	opts = [];
end

if (nargin < 4) | isempty(estype)
	estype = 3;			% error estimate type
end

if (nargin < 3) | isempty(H)
	H = speye(size(A,2));		% default regularization matrix
end
if iseye(H)
	Heye = 1;
else
	Heye = 0;
end

% set default options
if length(opts) < 1
	forceta = 0;			% do not force use of etanu
else
	forceta = opts(1);
end

if ~exist('tsvd','file')
	error('The "Regularization Tools" toolbox is not on the path.')
end

% compute csvd/cgsvd
if (nargin < 6)
	if Heye
		[U s V] = csvd(A);
	else
		[U s Z V ZI] = cgsvd(A,H);
	end
end

[m n] = size(A);
[p n1] = size(H);
if (n ~= n1) || (m ~= size(b,1))
	error('Sizes of arrays do not match.')
end
if m == n
	square = 1;
else
	square = 0;
end

if square && (forceta == 1)
%	warning('errestools:errestsv:ForceUseOfEtanu', ...
%			'Forcing use of etanu for a square matrix')
	square = 0;
end

if Heye
	[X norm_r] = tsvd(U,s,V,b,[1:n]');	% k=n,n-1,...,1
	beta = U'*b;
	sb = s.*beta;

	%norm_r = zeros(n,1);
	%norm_r(n:-1:1) = sqrt([0;cumsum(beta(n:-1:2).^2)]);
	%if m>n,  norm_r = sqrt(norm_r.^2 + norm(b-U*beta)^2);  end
	v = sb.*sb;
	norm_Atr = zeros(n,1);
	norm_Atr(n:-1:1) = sqrt([0;cumsum(v(n:-1:2))]);
	if square
		G = U'*V;
		rAr = zeros(n,1);
		for i = 2:n
			rAr(i-1) = beta(i:n)'*G(i:n,i:n)*sb(i:n);
		end
		rAr = abs(rAr);
		errest = enu( norm_r, rAr, norm_Atr, estype);
	else
		v = v.*s.*s;
		norm_AAtr = zeros(n,1);
		norm_AAtr(n:-1:1) = sqrt([0;cumsum(v(n:-1:2))]);
		if (forceta == 2)
			v = v.*s.*s;
			norm_AtAAtr = zeros(n,1);
			norm_AtAAtr(n:-1:1) = sqrt([0;cumsum(v(n:-1:2))]);
			errest = enu( norm_Atr, norm_AAtr, norm_AtAAtr, estype);
		else
			errest = etanu( norm_r, norm_Atr, norm_AAtr, estype);
		end
	end
else
	[X norm_r] = tgsvd(U,s,Z,b,[0:p]');	% k=p,p-1,...,0
	beta = U'*b;
	sb = [s(:,1);zeros(n-p,1)] .* beta;

	%norm_r = zeros(p+1,1);
	%norm_r(p+1:-1:1) = [0;sqrt(cumsum(beta(1:p).^2))];
	%if m>n,  norm_r = sqrt(norm_r.^2 + norm(b-U*beta)^2);  end

	W = zeros(n,p+1);
	W(:,p+1:-1:1) = [zeros(n,1) ZI'*triu(sb*ones(1,p))];
	norm_Atr = zeros(p+1,1);
	for i = 1:p+1
		norm_Atr(i) = norm(W(:,i));
	end

	if square
		rAr = zeros(p+1,1);
		W = U'*W;
		for i = 1:p+1
			rAr(i) = beta(1:p-i+1)' * W(1:p-i+1,i);
		end
		rAr = abs(rAr);
		errest = enu( norm_r, rAr, norm_Atr, estype);
	else
		W = spdiags([s(:,1);ones(n-p,1)],0,n,n)*(ZI*W);
		norm_AAtr = zeros(p+1,1);
		for i = 1:p+1
			norm_AAtr(i) = norm(W(:,i));
		end
		errest = etanu( norm_r, norm_Atr, norm_AAtr, estype);
	end

end

[mm kopt] = min(errest);
x = X(:,kopt);
if ~Heye,  kopt = kopt-1;  end



function flag = iseye(M)
%ISEYE	true for identity matrix
%
%	iseye(A) returns 1 is A is an identity matrix, 0 otherwise

%	Giuseppe Rodriguez,  31-01-08
%	Email: rodriguez@unica.it

[m,n]=size(M);
if (m==n) && isequal(M,speye(m))
	flag = 1;
else
	flag = 0;
end

