function eta = etanutik( norm_r, norm_Ex, norm_AEx, lam, nu)
%ETANUTIK error estimate for Tikhonov regularization (general case)
%
%   e = etanutik(norm_r,norm_Ex,norm_AEx,lam,nu) computes the error estimate
%   for a set of regularized solutions (x(:,i),i=1,...,k) of a linear system,
%   computed by minimizing the Tikhonov functional
%	|| A*x - b ||^2 + lambda^2 * || H*x ||^2
%
%   The input parameters norm_r, norm_Ex, norm_AEx and lam are k-vectors such
%   that
%	norm_r(i) = norm(r(:,i)), where r(:,i)=b-A*x(:,i)
%	norm_Ex(i) = norm(H'*H*x(:,i))
%	norm_AEx(i) = norm(A*H'*H*x(:,i))
%	lam(i) is the regularization parameter corresponding to x(:,i)
%   nu is the index of the estimate (real number, default 3).
%
%   See also etanu, enu, ehatnu, enutik.

% This function computes the estimates eta_nu defined in eq. (3.14) of the paper
%   C. Brezinski, G. Rodriguez, and S. Seatzu.
%   Error estimates for the regularization of least squares problems.
%   Numer. Algorithms, 2009. In press. 

%	Giuseppe Rodriguez,  12-05-08
% 	Email: rodriguez@unica.it

if nargin < 5,  nu = 3;  end
eta = (norm_r.^(nu-1)) .* (norm_Ex.^(5-2*nu)) .* (norm_AEx.^(nu-3)) ./ ...
							(lam.^(2*nu-4));

